import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;

/**
 * The JVMStats is a Jedit plugin which shows statistics of JVM memory usage.
 *
 * @author   <a href="mailto:uzi@landsmanns.com"Uzi Landsmann</a>
 */
public class JVMStats extends JPanel implements EBComponent, Runnable {
	private View view;
	private boolean floating;
	private boolean go = true;
	private Runtime rt = Runtime.getRuntime();

	/**
	 * Handles messages from JEDIT. Currently not use.
	 *
	 * @param message  the message to handle
	 */
	public void handleMessage(EBMessage message) { }

	/**
	 * Notifies this component that it now has a parent component. When this method
	 * is invoked, the chain of parent components is set up with KeyboardAction
	 * event listeners.
	 */
	public void addNotify() {
		super.addNotify();
		EditBus.addToBus(this);
	}

	/**
	 * Notifies this component that it no longer has a parent component. When this
	 * method is invoked, any KeyboardActions set up in the the chain of parent
	 * components are removed.
	 */
	public void removeNotify() {
		go = false;
		super.removeNotify();
		EditBus.removeFromBus(this);
	}

	/**
	 * Constructor for the JVMStats object
	 *
	 * @param view      the window to use
	 * @param position  the plugin position
	 */
	public JVMStats(View view, String position) {
		this.view = view;
		this.floating = position.equals(DockableWindowManager.FLOATING);

		setLayout(new FlowLayout(FlowLayout.CENTER));

		JPanel canvas = new JPanel();
		canvas.setLayout(new BoxLayout(canvas, BoxLayout.Y_AXIS));
		canvas.add(new MemoryPanel());

		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JButton gc = new JButton("Collect garbage");
		gc.addActionListener(
			new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					rt.gc();
				}
			});
		buttonPanel.add(gc);

		canvas.add(buttonPanel);
		add(canvas);

		new Thread(this).start();
	}

	/** Repaints the graph and the labels unitl it's time to stop. */
	public void run() {
		while(go) {
			repaint();
			try {
				Thread.sleep(1000);
			} catch(InterruptedException ie) {
				ie.printStackTrace();
			}
		}
	}

	/**
	 * The main JVMStats panel.
	 *
	 * @author   <a href="mailto:uzi-b-a@dsv.su.se">Uzi Landsmann</a>
	 */
	class MemoryPanel extends JPanel {

		private Vector mems = new Vector();
		private long mm = rt.maxMemory() / 1024;
		private int seconds = 10;
		private int width = 200;
		private int height = 160;
		private int gWidth = 190;
		private int gHeight = 100;
		private int gX = 10;
		private int gY = 55;
		private long y = gY + gHeight / 2;

		/** Constructor for the MemoryPanel object. */
		MemoryPanel() {
			setPreferredSize(new Dimension(width, height));
		}

		/**
		 * Paints the panel.
		 *
		 * @param g  the Graphics object to use when painting the panel
		 */
		public void paint(Graphics g) {
			long tm = rt.totalMemory() / 1024;
			long um = tm - rt.freeMemory() / 1024;
			long thno = Thread.activeCount();

			mems.add(new Long(um));
			if(mems.size() > seconds + 1)
				mems.remove(0);
			// memory information in text
			g.clearRect(0, 0, width, height);

			g.drawString("used memory: " + um + " of " + tm + " kb", 5, 15);
			g.drawString("max memory: " + mm + " kb", 5, 30);
			g.drawString("no of threads: " + thno, 5, 45);

			// memory graph
			g.setColor(Color.black);
			g.fillRect(gX, gY, gWidth, gHeight);
			g.setColor(Color.green);
			long x = gX;
			long tempX;
			long tempY;
			for(int i = 0; i < Math.min(mems.size(), seconds + 1); i++) {
				tempX = gX + gWidth / seconds * i;
				tempY = gY + (tm - ((Long)mems.get(i)).longValue()) * gHeight / tm;

				if(i != 0)
					g.drawLine((int)x, (int)y, (int)tempX, (int)tempY);

				x = tempX;
				y = tempY;
			}
		}
	}
}

